@@ -6,7 +6,7 @@ use Mouse;
 use Lemonldap::NG::Common::Conf::Constants;
 use Lemonldap::NG::Common::Conf::ReConstants;
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.7';
 
 extends 'Lemonldap::NG::Common::Conf::AccessLib';
 
@@ -195,9 +195,11 @@ sub virtualHosts {
                 type  => 'keyText',
             };
 
-            # If rule contains a comment, split it
+            # If rule contains a comment or an AuthLevel, split them
             if ( $query eq 'locationRules' ) {
                 $res->{comment} = '';
+                $res->{level} = '';
+                $res->{level} = $1 if ( $r =~ s/\(\?#AuthnLevel=(-?\d+)\)// );
                 if ( $r =~ s/\(\?#(.*?)\)// ) {
                     $res->{title} = $res->{comment} = $1;
                 }
@@ -267,7 +269,9 @@ sub _samlMetaDataNodes {
     my ( $id, $resp ) = ( 1, [] );
 
     # Return all exported attributes if asked
-    if ( $query =~ /^saml${type}MetaDataExportedAttributes$/ ) {
+    if ( $query =~
+        /^saml${type}MetaDataExportedAttributes|samlSPMetaDataMacros$/ )
+    {
         my $pk =
           eval { $self->getConfKey( $req, $query )->{$partner} } // {};
         return $self->sendError( $req, undef, 400 ) if ( $req->error );
@@ -378,7 +382,7 @@ sub _oidcMetaDataNodes {
 
     # Return all exported attributes if asked
     if ( $query =~
-        /^(?:oidc${type}MetaDataExportedVars|oidcRPMetaDataOptionsExtraClaims)$/
+/^(?:oidc${type}MetaDataExportedVars|oidcRPMetaDataOptionsExtraClaims|oidcRPMetaDataMacros)$/
       )
     {
         my $pk = eval { $self->getConfKey( $req, $query )->{$partner} } // {};
@@ -476,7 +480,7 @@ sub _casMetaDataNodes {
 
     # Return all exported attributes if asked
     if ( $query =~
-/^(?:cas${type}MetaDataExportedVars|casSrvMetaDataOptionsProxiedServices)$/
+/^(?:cas${type}MetaDataExportedVars|casSrvMetaDataOptionsProxiedServices|casAppMetaDataMacros)$/
       )
     {
         my $pk = eval { $self->getConfKey( $req, $query )->{$partner} } // {};
@@ -541,10 +545,16 @@ sub authChoiceModules {
         my @res;
         foreach my $k ( sort keys %$value ) {
             my $data = [ split /;/, $value->{$k} ];
-            eval { $data->[5] = from_json( $data->[5] ) if $data->[5] };
-            if ($@) {
-                $self->logger->error(
-                    "Bad value in choice over parameters, deleted ($@)");
+            if ( $data->[5] ) {
+                my $over;
+                eval { $over = from_json( $data->[5] ) };
+                if ($@) {
+                    $self->logger->error(
+                        "Bad value in choice over parameters, deleted ($@)");
+                }
+                else {
+                    $data->[5] = [ map { [ $_, $over->{$_} ] } keys %{$over} ];
+                }
             }
             push @res,
               {
@@ -800,8 +810,13 @@ sub getKey {
     unless ($key) {
         return $self->metadata($req);
     }
-    $self->userLogger->info(
-        'User ' . $self->userId($req) . " asks for key $key" );
+    if ( $self->can('userId') ) {
+        $self->userLogger->info(
+            'User ' . $self->userId($req) . " asks for key $key" );
+    }
+    else {
+        $self->logger->info("REST request to get configuration key $key");
+    }
     my $value = $self->getConfKey( $req, $key );
     return $self->sendError( $req, undef, 400 ) if ( $req->error );
 
