@@ -11,7 +11,7 @@ use Time::Local;
 use DBI;
 use Encode;
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.0.7';
 
 extends 'Lemonldap::NG::Common::Notifications';
 
@@ -93,9 +93,9 @@ sub get {
 }
 
 ## @method hashref getAll()
-# Return all messages not notified.
+# Return all pending notifications.
 # @return hashref where keys are internal reference and values are hashref with
-# keys date, uid and ref.
+# keys date, uid, ref and condition.
 sub getAll {
     my $self = shift;
     $self->_execute( 'SELECT * FROM '
@@ -114,6 +114,28 @@ sub getAll {
     return $result;
 }
 
+## @method hashref getExisting()
+# Return all notifications.
+# @return hashref where keys are internal reference and values are hashref with
+# keys date, uid, ref and condition.
+sub getExisting {
+    my $self = shift;
+    $self->_execute( 'SELECT * FROM '
+          . $self->dbiTable
+          . ' ORDER BY date' );
+    my $result;
+    while ( my $h = $self->sth->fetchrow_hashref() ) {
+        $result->{"$h->{date}#$h->{uid}#$h->{ref}"} = {
+            date      => $h->{date},
+            uid       => $h->{uid},
+            ref       => $h->{ref},
+            condition => $h->{condition}
+        };
+    }
+    $self->logger->warn( $self->sth->err() ) if ( $self->sth->err() );
+    return $result;
+}
+
 ## @method boolean delete(string myref)
 # Mark a notification as done.
 # @param $myref identifier returned by get() or getAll()
