@@ -9,7 +9,7 @@ use strict;
 use Mouse;
 use MIME::Base64;
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.7';
 
 extends 'Lemonldap::NG::Common::Notifications';
 
@@ -58,9 +58,9 @@ sub get {
 }
 
 ## @method hashref getAll()
-# Return all messages not notified.
+# Return all pending notifications.
 # @return hashref where keys are internal reference and values are hashref with
-# keys date, uid and ref.
+# keys date, uid, ref and condition.
 sub getAll {
     my $self = shift;
     opendir D, $self->{dirName};
@@ -82,6 +82,31 @@ sub getAll {
     return \%h;
 }
 
+## @method hashref getExisting()
+# Return all notifications.
+# @return hashref where keys are internal reference and values are hashref with
+# keys date, uid, ref and condition.
+sub getExisting {
+    my $self = shift;
+    opendir D, $self->{dirName};
+    my @notif;
+    my $fns = $self->{fileNameSeparator};
+    @notif = grep /^\S*\.$ext$/, readdir(D);
+    my %h = map {
+/^(\d{8})${fns}([^\s${fns}]+)${fns}([^\s${fns}]+)(?:${fns}([^\s${fns}]+))?\.(?:$ext|done)$/
+          ? (
+            $_ => {
+                date      => $1,
+                uid       => $2,
+                ref       => decode_base64($3),
+                condition => decode_base64( $4 // '' )
+            }
+          )
+          : ()
+    } @notif;
+    return \%h;
+}
+
 ## @method boolean delete(string myref)
 # Mark a notification as done.
 # @param $myref identifier returned by get() or getAll()
