@@ -4,7 +4,7 @@ use strict;
 use Mouse;
 use XML::LibXML;
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.7';
 
 # XML parser
 has parser => (
@@ -18,7 +18,8 @@ has parser => (
 # @param $xml XML string containing notification
 # @return number of notifications done
 sub newNotification {
-    my ( $self, $xml ) = @_;
+    my ( $self, $xml, $defaultCond ) = @_;
+    $defaultCond ||= '';
     eval { $xml = $self->parser->parse_string($xml) };
     if ( my $err = $@ ) {
         eval { $self->logger->error("Unable to read XML file : $err") };
@@ -44,6 +45,9 @@ sub newNotification {
                 $self->logger->error("$err");
                 return 0;
             }
+            
+            # Prevent to store time. Keep date only
+            $tmp =~ s/^(\d{4}-\d{2}-\d{2}).*$/$1/;
             push @data, $tmp;
         }
 
@@ -53,7 +57,12 @@ sub newNotification {
             if ( $tmp = $notif->getAttribute($_) ) {
                 push @data, $tmp;
             }
-            else { push @data, ""; }
+            else {
+                $self->userLogger->info(
+                    "Set defaultCondition ($defaultCond) for notification " . $notif->{reference}
+                );
+                push @data, $defaultCond;
+            }
         }
 
         my $result = XML::LibXML::Document->new( $version, $encoding );
