@@ -1,6 +1,6 @@
 package Lemonldap::NG::Handler::Main::Reload;
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.0.7';
 
 package Lemonldap::NG::Handler::Main;
 
@@ -13,6 +13,7 @@ use Scalar::Util qw(weaken);
 
 use constant UNPROTECT => 1;
 use constant SKIP      => 2;
+use constant MAYSKIP   => 3;
 
 our @_onReload;
 
@@ -34,7 +35,7 @@ sub onReload {
 sub checkConf {
     my ( $class, $force ) = @_;
     $class->logger->debug("Check configuration for $class");
-    my $prm = { local => !$force, localPrm => $class->localConfig };
+    my $prm  = { local => !$force, localPrm => $class->localConfig };
     my $conf = $class->confAcc->getConf($prm);
     chomp $Lemonldap::NG::Common::Conf::msg;
 
@@ -179,8 +180,9 @@ sub jailInit {
     my ( $class, $conf ) = @_;
 
     $class->tsv->{jail} = Lemonldap::NG::Handler::Main::Jail->new( {
-            useSafeJail     => $conf->{useSafeJail},
-            customFunctions => $conf->{customFunctions},
+            useSafeJail          => $conf->{useSafeJail},
+            customFunctions      => $conf->{customFunctions},
+            multiValuesSeparator => $conf->{multiValuesSeparator},
         }
     );
     $class->tsv->{jail}->build_jail( $class, $conf->{require} );
@@ -197,6 +199,8 @@ sub defaultValuesInit {
         securedCookie           timeout            timeoutActivity
         timeoutActivityInterval useRedirectOnError useRedirectOnForbidden
         useSafeJail             whatToTrace        handlerInternalCache
+        handlerServiceTokenTTL  customToTrace      lwpOpts lwpSslOpts
+        authChoiceParam         authChoiceAuthBasic
         )
     );
 
@@ -229,6 +233,8 @@ sub defaultValuesInit {
               $conf->{vhostOptions}->{$vhost}->{vhostType};
             $class->tsv->{authnLevel}->{$vhost} =
               $conf->{vhostOptions}->{$vhost}->{vhostAuthnLevel};
+            $class->tsv->{serviceTokenTTL}->{$vhost} =
+              $conf->{vhostOptions}->{$vhost}->{vhostServiceTokenTTL};
         }
     }
     return 1;
@@ -272,7 +278,13 @@ sub locationRulesInit {
 
     foreach my $vhost ( keys %$orules ) {
         my $rules = $orules->{$vhost};
-        $class->tsv->{locationCount}->{$vhost} = 0;
+        $class->tsv->{locationCount}->{$vhost}         = 0;
+        $class->tsv->{locationCondition}->{$vhost}     = [];
+        $class->tsv->{locationProtection}->{$vhost}    = [];
+        $class->tsv->{locationRegexp}->{$vhost}        = [];
+        $class->tsv->{locationConditionText}->{$vhost} = [];
+        $class->tsv->{locationAuthnLevel}->{$vhost}    = [];
+
         foreach my $url ( sort keys %{$rules} ) {
             my ( $cond, $prot ) = $class->conditionSub( $rules->{$url} );
             unless ($cond) {
@@ -291,17 +303,21 @@ sub locationRulesInit {
                 push @{ $class->tsv->{locationCondition}->{$vhost} },  $cond;
                 push @{ $class->tsv->{locationProtection}->{$vhost} }, $prot;
                 push @{ $class->tsv->{locationRegexp}->{$vhost} },     qr/$url/;
+                push @{ $class->tsv->{locationAuthnLevel}->{$vhost} },
+                  $url =~ /\(\?#AuthnLevel=(-?\d+)\)/
+                  ? $1
+                  : undef;
                 push @{ $class->tsv->{locationConditionText}->{$vhost} },
-                    $cond =~ /^\(\?#(.*?)\)/ ? $1
-                  : $cond =~ /^(.*?)##(.+)$/ ? $2
-                  :                            $url;
+                    $url =~ /^\(\?#(.*?)\)/ ? $1
+                  : $url =~ /^(.*?)##(.+)$/ ? $2
+                  :                           $url;
                 $class->tsv->{locationCount}->{$vhost}++;
             }
         }
 
         # Default policy set to 'accept'
         unless ( $class->tsv->{defaultCondition}->{$vhost} ) {
-            $class->tsv->{defaultCondition}->{$vhost} = sub { 1 };
+            $class->tsv->{defaultCondition}->{$vhost}  = sub { 1 };
             $class->tsv->{defaultProtection}->{$vhost} = 0;
         }
     }
@@ -314,6 +330,8 @@ sub locationRulesInit {
 # @param $args reference to the configuration hash
 sub sessionStorageInit {
     my ( $class, $conf ) = @_;
+
+    # Global session storage
     unless ( $class->tsv->{sessionStorageModule} = $conf->{globalStorage} ) {
         $class->logger->error("globalStorage required");
         return 0;
@@ -322,6 +340,20 @@ sub sessionStorageInit {
     die($@) if ($@);
     $class->tsv->{sessionStorageOptions} = $conf->{globalStorageOptions};
 
+    # OIDC session storage
+    if ( $conf->{oidcStorage} ) {
+        eval "use " . $conf->{oidcStorage};
+        die($@) if ($@);
+        $class->tsv->{oidcStorageModule}  = $conf->{oidcStorage};
+        $class->tsv->{oidcStorageOptions} = $conf->{oidcStorageOptions};
+
+    }
+    else {
+        $class->tsv->{oidcStorageModule}  = $conf->{globalStorage};
+        $class->tsv->{oidcStorageOptions} = $conf->{globalStorageOptions};
+    }
+
+    # Local session storage
     if ( $conf->{localSessionStorage} ) {
         $class->tsv->{sessionCacheModule} = $conf->{localSessionStorage};
         $class->tsv->{sessionCacheOptions} =
@@ -331,11 +363,13 @@ sub sessionStorageInit {
         if ( $conf->{status} ) {
             my $params = "";
             if ( $class->tsv->{sessionCacheModule} ) {
-                require Data::Dumper;
-                $params = ' '
-                  . $class->tsv->{sessionCacheModule} . ','
-                  . Data::Dumper->new( [ $class->tsv->{sessionCacheOptions} ] )
-                  ->Terse(1)->Indent(0)->Dump;    # To send params on one line
+                $params = ' ' . join(
+                    ',',
+                    $class->tsv->{sessionCacheModule} . map {
+                        "$_ => "
+                          . $class->tsv->{sessionCacheOptions}->{$_}
+                    } keys %{ $class->tsv->{sessionCacheOptions} // {} }
+                );
             }
             $class->tsv->{statusPipe}->print("RELOADCACHE $params\n");
         }
@@ -361,7 +395,8 @@ sub headersInit {
         $class->tsv->{headerList}->{$vhost} = [ keys %headers ];
         my $sub = '';
         foreach ( keys %headers ) {
-            my $val = $class->substitute( $headers{$_} );
+            $headers{$_} ||= "''";
+            my $val = $class->substitute( $headers{$_} ) . " // ''";
             $sub .= "('$_' => $val),";
         }
 
@@ -369,7 +404,7 @@ sub headersInit {
             $class->buildSub($sub) )
         {
             $class->tsv->{maintenance}->{$vhost} = 1;
-            $class->logger->error( "$class Unable to forge headers: "
+            $class->logger->error( "$class Unable to forge $vhost headers: "
                   . $class->tsv->{jail}->error );
         }
     }
@@ -423,6 +458,7 @@ sub postUrlInit {
 # @return array (ref(sub), int)
 sub conditionSub {
     my ( $class, $cond ) = @_;
+    $cond =~ s/\(\?#(\d+)\)$//;
     my ( $OK, $NOK ) = ( sub { 1 }, sub { 0 } );
 
     # Simple cases : accept and deny
@@ -511,6 +547,9 @@ sub conditionSub {
         );
     }
 
+    my $mayskip = 0;
+    $mayskip = MAYSKIP if $cond =~ /\bskip\b/;
+
     # Replace some strings in condition
     $cond = $class->substitute($cond);
     my $sub;
@@ -520,7 +559,7 @@ sub conditionSub {
     }
 
     # Return sub and protected flag
-    return ( $sub, 0 );
+    return ( $sub, $mayskip );
 }
 
 ## @method arrayref aliasInit
@@ -551,9 +590,10 @@ sub substitute {
     $expr =~ s/\$ip\b/\$ENV{REMOTE_ADDR}/sg;
 
     # substitute vars with session data, excepts special vars $_ and $\d+
-    $expr =~ s/\$(?!(?:ENV|env)\b)([_a-zA-Z]\w*)/\$s->{$1}/sg;
+    $expr =~ s/\$(?!(?:ENV|env)\b)(_\w+|[a-zA-Z]\w*)/\$s->{$1}/sg;
     $expr =~ s/\$ENV\{/\$r->{env}->\{/g;
     $expr =~ s/\$env->\{/\$r->{env}->\{/g;
+    $expr =~ s/\bskip\b/q\{999_SKIP\}/g;
 
     return $expr;
 }
