@@ -1,7 +1,7 @@
 # Main running methods file
 package Lemonldap::NG::Handler::Main::Run;
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.0.7';
 
 package Lemonldap::NG::Handler::Main;
 
@@ -83,8 +83,11 @@ sub checkType {
     my ( $class, $req ) = @_;
 
     if ( time() - $class->lastCheck > $class->checkTime ) {
-        die("$class: No configuration found")
-          unless ( $class->checkConf );
+        unless ( $class->checkConf ) {
+            $class->logger->error("$class: No configuration found");
+            $req->data->{noTry} = 1;
+            return 'Fail';
+        }
     }
     my $vhost = $class->resolveAlias($req);
     return ( defined $class->tsv->{type}->{$vhost} )
@@ -94,10 +97,10 @@ sub checkType {
 
 ## @rmethod int run
 # Check configuration and launch Lemonldap::NG::Handler::Main::run().
-# Each $checkTime, the Apache child verify if its configuration is the same
+# Each $checkTime, server child verifies if its configuration is the same
 # as the configuration stored in the local storage.
 # @param $rule optional Perl expression to grant access
-# @return Apache constant
+# @return constant
 
 sub run {
     my ( $class, $req, $rule, $protection ) = @_;
@@ -118,11 +121,10 @@ sub run {
         }
     }
 
-    # Cross domain authentication
+    # Authentication process
     my $uri = $req->{env}->{REQUEST_URI};
-
-    $uri = $req->{env}->{REQUEST_URI};
     my ($cond);
+
     ( $cond, $protection ) = $class->conditionSub($rule) if ($rule);
     $protection = $class->isUnprotected( $req, $uri ) || 0
       unless ( defined $protection );
@@ -147,6 +149,9 @@ sub run {
 
         # ACCOUNTING (1. Inform web server)
         $class->set_user( $req, $session->{ $class->tsv->{whatToTrace} } );
+        $class->set_custom( $req, $session->{ $class->tsv->{customToTrace} } )
+          if $class->tsv->{customToTrace}
+          and $session->{ $class->tsv->{customToTrace} };
 
         # AUTHORIZATION
         return ( $class->forbidden( $req, $session ), $session )
@@ -187,6 +192,11 @@ sub run {
         $class->cleanHeaders($req);
         return $class->OK;
     }
+    elsif ( $protection == $class->MAYSKIP
+        and $class->grant( $req, $session, $uri, $cond ) eq '999_SKIP' )
+    {
+        return $class->OK;
+    }
 
     else {
 
@@ -262,11 +272,38 @@ sub checkMaintenanceMode {
 # @return True if the user is granted to access to the current URL
 sub grant {
     my ( $class, $req, $session, $uri, $cond, $vhost ) = @_;
+    my $level;
+
     return $cond->( $req, $session ) if ($cond);
 
     $vhost ||= $class->resolveAlias($req);
-    if ( my $level = $class->tsv->{authnLevel}->{$vhost} ) {
+
+    # Using URL authentification level if exists
+    for (
+        my $i = 0 ;
+        $i < ( $class->tsv->{locationCount}->{$vhost} || 0 ) ;
+        $i++
+      )
+    {
+        if ( $uri =~ $class->tsv->{locationRegexp}->{$vhost}->[$i] ) {
+            $level = $class->tsv->{locationAuthnLevel}->{$vhost}->[$i];
+            last;
+        }
+    }
+    $level
+      ? $class->logger->debug(
+        'Found AuthnLevel=' . $level . ' for "' . "$vhost$uri" . '"' )
+      : $class->logger->debug("No URL authentication level found...");
+
+    # Using VH authentification level if exists
+    if ( $level ||= $class->tsv->{authnLevel}->{$vhost} ) {
         if ( $session->{authenticationLevel} < $level ) {
+            $class->logger->debug(
+                "User authentication level = $session->{authenticationLevel}");
+            $class->logger->debug("Required authentication level = $level");
+            $class->logger->warn(
+"User rejected due to insufficient authentication level -> Session upgrade enabled"
+            );
             $session->{_upgrade} = 1;
             return 0;
         }
@@ -408,7 +445,7 @@ sub fetchId {
     my $value =
       $lookForHttpCookie
       ? ( $t =~ /${cn}http=([^,; ]+)/o ? $1 : 0 )
-      : ( $t =~ /$cn=([^,; ]+)/o ? $1 : 0 );
+      : ( $t =~ /$cn=([^,; ]+)/o       ? $1 : 0 );
 
     if ( $value && $lookForHttpCookie && $class->tsv->{securedCookie} == 3 ) {
         $value = $class->tsv->{cipher}->decryptHex( $value, "http" );
@@ -417,7 +454,7 @@ sub fetchId {
         $value = $class->tsv->{cipher}->decrypt($value);
         unless ( $value =~ s/^(.*)? (.*)$/$1/ and $2 eq $vhost ) {
             $class->userLogger->error(
-                "Bad CDA cookie: available for $2 instead od $vhost");
+                "Bad CDA cookie: available for $2 instead of $vhost");
             return undef;
         }
     }
@@ -625,7 +662,7 @@ sub isUnprotected {
 }
 
 ## @rmethod void sendHeaders()
-# Launch function compiled by forgeHeadersInit() for the current virtual host
+# Launch function compiled by headersInit() for the current virtual host
 sub sendHeaders {
     my ( $class, $req, $session ) = @_;
     my $vhost = $class->resolveAlias($req);
@@ -646,6 +683,27 @@ sub sendHeaders {
     }
 }
 
+## @rfunction array ref checkHeaders()
+# Return computed headers by headersInit() for the current virtual host
+# [ { key => 'header1', value => 'value1' }, { key => 'header2', value => 'value2' }, ...]
+sub checkHeaders {
+    my ( $class, $req, $session ) = @_;
+    my $vhost         = $class->resolveAlias($req);
+    my $array_headers = [];
+    if ( defined $class->tsv->{forgeHeaders}->{$vhost} ) {
+
+        # Create array of hashes with headers
+        my %headers =
+          $class->tsv->{forgeHeaders}->{$vhost}->( $req, $session );
+        foreach my $h ( sort keys %headers ) {
+            defined $headers{$h}
+              ? push @$array_headers, { key => $h, value => $headers{$h} }
+              : push @$array_headers, { key => $h, value => '' };
+        }
+    }
+    return $array_headers;
+}
+
 ## @rmethod void cleanHeaders()
 # Unset HTTP headers, when sendHeaders is skipped
 sub cleanHeaders {
@@ -759,7 +817,11 @@ sub postInputFilter {
         my %data =
           $class->tsv->{inputPostData}->{$vhost}->{$uri}->( $req, $session );
         foreach ( keys %data ) {
-            $data{$_} = uri_escape( $data{$_} );
+            my $post_key   = uri_escape($_);
+            my $post_value = uri_escape( $data{$_} );
+            delete $data{$_};
+            $data{$post_key} = $post_value;
+            $class->logger->debug("Send key $post_key with value $post_value");
         }
         $class->setPostParams( $req, \%data );
     }
@@ -788,7 +850,8 @@ sub postJavascript {
       : "form.submit();\n";
 
     my $jqueryUrl = $formParams->{jqueryUrl} || "";
-    $jqueryUrl = &{ $class->tsv->{portal} } . "skins/common/js/jquery-1.10.2.js"
+    $jqueryUrl =
+      &{ $class->tsv->{portal} } . "static/bwr/jquery/dist/jquery.min.js"
       if ( $jqueryUrl eq "default" );
     $jqueryUrl = "<script type='text/javascript' src='$jqueryUrl'></script>\n"
       if ($jqueryUrl);
