@@ -1,6 +1,6 @@
 package Lemonldap::NG::Manager::Conf::Zero;
 
-our $VERSION = '2.0.1';
+our $VERSION = '2.0.6';
 
 sub zeroConf {
     my ( $domain, $sessionDir, $persistentSessionDir, $notificationDir ) = @_;
@@ -66,7 +66,7 @@ sub zeroConf {
                         'uri'         => 'http://lemonldap-ng.org/',
                         'display'     => 'on',
                         'logo'        => 'network.png',
-                        'name'        => 'Offical Website'
+                        'name'        => 'Official Website'
                     },
                     'type' => 'application'
                 },
@@ -108,7 +108,7 @@ sub zeroConf {
         },
         'macros' => {
             '_whatToTrace' =>
-'$_auth eq \'SAML\' ? "$_user\\@$_idpConfKey" : $_auth eq \'OpenIDConnect\' ? "$_user\\@$_oidcConnectedRP" : "$_user"',
+'$_auth eq \'SAML\' ? lc($_user.\'@\'.$_idpConfKey) : $_auth eq \'OpenIDConnect\' ? lc($_user.\'@\'.$_oidc_OP) : lc($_user)',
             'UA' => '$ENV{HTTP_USER_AGENT}'
         },
         'notificationStorageOptions' => {
@@ -147,6 +147,11 @@ sub zeroConf {
         'portal'              => "http://auth.$domain/",
         'notificationStorage' => 'File',
         'locationRules'       => {
+            "auth.$domain" => {
+                '(?#checkUser)^/checkuser' => '$uid eq "dwho"',
+                '(?#errors)^/lmerror/'     => 'accept',
+                'default'                  => 'accept'
+            },
             "test1.$domain" => {
                 'default'  => 'accept',
                 '^/logout' => 'logout_sso'
@@ -157,9 +162,11 @@ sub zeroConf {
             },
             "manager.$domain" => {
                 'default' => '$uid eq "dwho" or $uid eq "rtyler"',
-                '(?#Configuration)^/(manager\.html|conf/)' => '$uid eq "dwho"',
-                '(?#Sessions)/sessions' => '$uid eq "dwho" or $uid eq "rtyler"',
-                '(?#Notifications)/notifications' =>
+'(?#Configuration)^/(.*?\.(fcgi|psgi)/)?(manager\.html|confs/|$)'
+                  => '$uid eq "dwho"',
+                '(?#Sessions)/(.*?\.(fcgi|psgi)/)?sessions' =>
+                  '$uid eq "dwho" or $uid eq "rtyler"',
+                '(?#Notifications)/(.*?\.(fcgi|psgi)/)?notifications' =>
                   '$uid eq "dwho" or $uid eq "rtyler"',
             }
         },
