@@ -9,13 +9,13 @@ use Mouse;
 use JSON qw(from_json to_json);
 use Lemonldap::NG::Portal::Main::Constants qw(
   PE_ERROR
-  PE_BADCREDENTIALS
+  PE_BADOTP
   PE_FORMEMPTY
   PE_OK
   PE_SENDRESPONSE
 );
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.6';
 
 extends 'Lemonldap::NG::Portal::Main::SecondFactor';
 
@@ -96,7 +96,7 @@ sub run {
         $self->userLogger->warn( 'User '
               . $req->{sessionInfo}->{ $self->conf->{whatToTrace} }
               . ' has no Yubikey registered' );
-        return PE_BADCREDENTIALS;
+        return PE_BADOTP;
     }
     $self->logger->debug("Found Yubikey : $yubikey");
 
@@ -108,7 +108,7 @@ sub run {
             MAIN_LOGO   => $self->conf->{portalMainLogo},
             SKIN        => $self->p->getSkin($req),
             TOKEN       => $token,
-            TARGET      => '/yubikey2fcheck',
+            TARGET      => '/yubikey2fcheck?skin=' . $self->p->getSkin($req),
             INPUTLOGO   => 'yubikey.png',
             LEGEND      => 'clickOnYubikey',
             CHECKLOGINS => $checkLogins
@@ -149,11 +149,11 @@ sub verify {
       )
     {
         $self->userLogger->warn('Yubikey not registered');
-        return PE_BADCREDENTIALS;
+        return PE_BADOTP;
     }
     if ( $self->yubi->otp($code) ne 'OK' ) {
         $self->userLogger->warn('Yubikey verification failed');
-        return PE_BADCREDENTIALS;
+        return PE_BADOTP;
     }
     PE_OK;
 }
