@@ -14,7 +14,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_SENDRESPONSE
 );
 
-our $VERSION = '2.0.3';
+our $VERSION = '2.0.6';
 
 extends 'Lemonldap::NG::Portal::Main::Issuer',
   'Lemonldap::NG::Portal::Lib::CAS';
@@ -85,7 +85,11 @@ sub storeEnvAndCheckGateway {
         $self->logger->debug(
             "Gateway mode requested, redirect without authentication");
         $req->response( [ 302, [ Location => $service ], [] ] );
-        $req->pdata( {} );
+        for my $s ( $self->ipath, $self->ipath . 'Path' ) {
+            $self->logger->debug("Removing $s from pdata")
+              if delete $req->pdata->{$s};
+        }
+
         return PE_SENDRESPONSE;
     }
 
@@ -187,10 +191,10 @@ sub run {
                 if ( $rule->( $req, $req->sessionInfo ) ) {
                     $self->logger->debug("CAS service $service access allowed");
                 }
-
                 else {
-                    $self->userLogger->error(
-                        "CAS service $service access not allowed");
+                    $self->userLogger->warn( 'User '
+                          . $req->sessionInfo->{ $self->conf->{whatToTrace} }
+                          . " is not authorized to access to $service" );
 
                     if ( $casAccessControlPolicy =~ /^(error)$/i ) {
                         $self->logger->debug(
@@ -208,11 +212,15 @@ sub run {
             }
         }
 
+        $self->userLogger->notice( 'User '
+              . $req->sessionInfo->{ $self->conf->{whatToTrace} }
+              . " is authorized to access to $service" );
+
         unless ($casServiceTicket) {
 
             # Check last authentication time to decide if
             # the authentication is recent or not
-            my $casRenewFlag = 0;
+            my $casRenewFlag     = 0;
             my $last_authn_utime = $req->{sessionInfo}->{_lastAuthnUTime} || 0;
             if (
                 time() - $last_authn_utime <
@@ -793,9 +801,21 @@ sub _validate2 {
     }
 
     foreach my $casAttribute ( keys %$ev ) {
-        my $localSessionValue = $localSession->data->{ $ev->{$casAttribute} };
-        $attributes->{$casAttribute} = $localSessionValue
-          if defined $localSessionValue;
+        my $sessionAttr = $ev->{$casAttribute};
+        my $value;
+
+        # Lookup per-service macros first, and then local sessions
+        #
+        if ( $app and $self->spMacros->{$app}->{$sessionAttr} ) {
+            $value = $self->spMacros->{$app}->{$sessionAttr}
+              ->( $req, $localSession->data );
+        }
+        else {
+            $value = $localSession->data->{$sessionAttr};
+        }
+
+        $attributes->{$casAttribute} = $value
+          if defined $value;
     }
 
     # Return success message
