@@ -19,7 +19,7 @@ use Mouse;
 
 use Lemonldap::NG::Portal::Main::Constants qw(PE_OK PE_REDIRECT);
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.7';
 
 # OpenID Connect standard claims
 use constant PROFILE => [
@@ -37,6 +37,7 @@ has oidcOPList   => ( is => 'rw', default => sub { {} }, );
 has oidcRPList   => ( is => 'rw', default => sub { {} }, );
 has rpAttributes => ( is => 'rw', default => sub { {} }, );
 has spRules      => ( is => 'rw', default => sub { {} } );
+has spMacros     => ( is => 'rw', default => sub { {} } );
 
 # return LWP::UserAgent object
 has ua => (
@@ -73,6 +74,7 @@ sub loadOPs {
     {
         $self->logger->warn(
             "No OpenID Connect Provider found in configuration");
+        return 1;
     }
 
     # Extract JSON data
@@ -97,6 +99,7 @@ sub loadRPs {
     {
         $self->logger->warn(
             "No OpenID Connect Relying Party found in configuration");
+        return 1;
     }
     $self->oidcRPList( $self->conf->{oidcRPMetaDataOptions} );
     foreach my $rp ( keys %{ $self->oidcRPList } ) {
@@ -130,6 +133,22 @@ sub loadRPs {
             }
             $self->spRules->{$rp} = $rule;
         }
+
+        # Load per-RP macros
+        my $macros = $self->conf->{oidcRPMetaDataMacros}->{$rp};
+        for my $macroAttr ( keys %{$macros} ) {
+            my $macroRule = $macros->{$macroAttr};
+            if ( length $macroRule ) {
+                $macroRule = $self->p->HANDLER->substitute($macroRule);
+                unless ( $macroRule = $self->p->HANDLER->buildSub($macroRule) )
+                {
+                    $self->error( 'OIDC RP macro error: '
+                          . $self->p->HANDLER->tsv->{jail}->error );
+                    return 0;
+                }
+                $self->spMacros->{$rp}->{$macroAttr} = $macroRule;
+            }
+        }
     }
     return 1;
 }
@@ -564,7 +583,7 @@ sub checkIDTokenValidity {
             $self->logger->error("Auth time was not returned by OP $op");
             return 0;
         }
-        if ( $auth_time + $max_age > time ) {
+        if ( time > $auth_time + $max_age ) {
             $self->userLogger->error(
 "Authentication time ($auth_time) is too old (Max age: $max_age)"
             );
@@ -628,15 +647,133 @@ sub decodeJSON {
     return $json_hash;
 }
 
-# Try to recover the OpenID Connect session corresponding to id and return session
-# If id is set to undef, return a new session
-# @return Lemonldap::NG::Common::Session object
-sub getOpenIDConnectSession {
-    my ( $self, $id, $info ) = @_;
+# Create a new Authorization Code
+# @param info hashref of session info
+# @return new Lemonldap::NG::Common::Session object
+
+sub newAuthorizationCode {
+    my ( $self, $rp, $info ) = @_;
+
+    return $self->getOpenIDConnectSession(
+        undef,
+        "authorization_code",
+        $self->conf->{oidcRPMetaDataOptions}->{$rp}
+          ->{oidcRPMetaDataOptionsAuthorizationCodeExpiration}
+          || $self->conf->{oidcServiceAuthorizationCodeExpiration},
+        ,
+        $info
+    );
+}
+
+# Get existing Authorization Code
+# @param id
+# @return new Lemonldap::NG::Common::Session object
+
+sub getAuthorizationCode {
+    my ( $self, $id ) = @_;
+
+    return $self->getOpenIDConnectSession( $id, "authorization_code" );
+}
+
+# Create a new Access Token
+# @param info hashref of session info
+# @return new Lemonldap::NG::Common::Session object
+
+sub newAccessToken {
+    my ( $self, $rp, $info ) = @_;
+
+    return $self->getOpenIDConnectSession(
+        undef,
+        "access_token",
+        $self->conf->{oidcRPMetaDataOptions}->{$rp}
+          ->{oidcRPMetaDataOptionsAccessTokenExpiration}
+          || $self->conf->{oidcServiceAccessTokenExpiration},
+        $info
+    );
+}
+
+# Get existing Access Token
+# @param id
+# @return new Lemonldap::NG::Common::Session object
+
+sub getAccessToken {
+    my ( $self, $id ) = @_;
+
+    return $self->getOpenIDConnectSession( $id, "access_token" );
+}
+
+# Create a new Refresh Token
+# @param info hashref of session info
+# @return new Lemonldap::NG::Common::Session object
+
+sub newRefreshToken {
+    my ( $self, $rp, $info, $offline ) = @_;
+    my $ttl =
+      $offline
+      ? ( $self->conf->{oidcRPMetaDataOptions}->{$rp}
+          ->{oidcRPMetaDataOptionsOfflineSessionExpiration}
+          || $self->conf->{oidcServiceOfflineSessionExpiration} )
+      : $self->conf->{timeout};
+
+    return $self->getOpenIDConnectSession( undef, "refresh_token", $ttl,
+        $info );
+}
+
+# Get existing Refresh Token
+# @param id
+# @return new Lemonldap::NG::Common::Session object
+
+sub getRefreshToken {
+    my ( $self, $id ) = @_;
+
+    return $self->getOpenIDConnectSession( $id, "refresh_token" );
+}
+
+sub updateRefreshToken {
+    my ( $self, $id, $infos ) = @_;
+
     my %storage = (
         storageModule        => $self->conf->{oidcStorage},
         storageModuleOptions => $self->conf->{oidcStorageOptions},
     );
+
+    unless ( $storage{storageModule} ) {
+        %storage = (
+            storageModule        => $self->conf->{globalStorage},
+            storageModuleOptions => $self->conf->{globalStorageOptions},
+        );
+    }
+
+    my $oidcSession = Lemonldap::NG::Common::Session->new( {
+            %storage,
+            cacheModule        => $self->conf->{localSessionStorage},
+            cacheModuleOptions => $self->conf->{localSessionStorageOptions},
+            id                 => $id,
+            info               => $infos,
+        }
+    );
+
+    if ( $oidcSession->error ) {
+        $self->userLogger->warn(
+            "OpenIDConnect session $id isn't yet available");
+        return undef;
+    }
+
+    return $oidcSession;
+}
+
+# Try to recover the OpenID Connect session corresponding to id and return session
+# If id is set to undef, return a new session
+# @return Lemonldap::NG::Common::Session object
+sub getOpenIDConnectSession {
+    my ( $self, $id, $type, $ttl, $info ) = @_;
+    my %storage = (
+        storageModule        => $self->conf->{oidcStorage},
+        storageModuleOptions => $self->conf->{oidcStorageOptions},
+    );
+
+    $ttl ||= $self->conf->{timeout};
+
     unless ( $storage{storageModule} ) {
         %storage = (
             storageModule        => $self->conf->{globalStorage},
@@ -650,7 +787,17 @@ sub getOpenIDConnectSession {
             cacheModuleOptions => $self->conf->{localSessionStorageOptions},
             id                 => $id,
             kind               => $self->sessionKind,
-            ( $info ? ( info => $info ) : () ),
+            (
+                $info
+                ? (
+                    info => {
+                        _type  => $type,
+                        _utime => time + $ttl - $self->conf->{timeout},
+                        %{$info}
+                    }
+                  )
+                : ()
+            ),
         }
     );
 
@@ -666,6 +813,35 @@ sub getOpenIDConnectSession {
         return undef;
     }
 
+    if ( $id and $type ) {
+        my $storedType = $oidcSession->{data}->{_type};
+
+        # Only check if a type is set in DB, for backward compatibility
+        if ( $storedType and $type ne $storedType ) {
+            $self->logger->error( "Wrong OpenID session type: "
+                  . $oidcSession->{data}->{_type}
+                  . ". Expected: "
+                  . $type );
+            return undef;
+        }
+
+        # Make sure the token is still valid, we already compensated for
+        # different TTLs when storing _utime
+        if (
+            time > ( $oidcSession->{data}->{_utime} + $self->conf->{timeout} ) )
+        {
+            $self->logger->error("Session $id has expired");
+            return undef;
+        }
+    }
+
+    # Make sure the token is still valid, we already compensated for
+    # different TTLs when storing _utime
+    if ( time > ( $oidcSession->{data}->{_utime} + $self->conf->{timeout} ) ) {
+        $self->logger->error("Session $id has expired");
+        return undef;
+    }
+
     return $oidcSession;
 }
 
@@ -766,7 +942,9 @@ sub verifyJWTSignature {
                   . " is present but algorithm is 'none'" );
             return 0;
         }
-        return 1;
+        $self->logger->debug(
+            "JWT algorithm is 'none', signature cannot be verified");
+        return 0;
     }
 
     if ( $alg eq "HS256" or $alg eq "HS384" or $alg eq "HS512" ) {
@@ -976,8 +1154,8 @@ sub createHash {
 # @param fragment Set to true to return fragment component
 # @return void
 sub returnRedirectError {
-    my ( $self, $req, $redirect_url, $error, $error_description, $error_uri,
-        $state, $fragment )
+    my ( $self, $req, $redirect_url, $error, $error_description,
+        $error_uri, $state, $fragment )
       = @_;
 
     my $urldc =
@@ -1027,6 +1205,56 @@ sub returnBearerError {
     ];
 }
 
+sub checkEndPointAuthenticationCredentials {
+    my ( $self, $req ) = @_;
+
+    # Check authentication
+    my ( $client_id, $client_secret ) =
+      $self->getEndPointAuthenticationCredentials($req);
+
+    unless ($client_id) {
+        $self->logger->error(
+"No authentication provided to get token, or authentication type not supported"
+        );
+        return undef;
+    }
+
+    # Verify that client_id is registered in configuration
+    my $rp = $self->getRP($client_id);
+
+    unless ($rp) {
+        $self->userLogger->error(
+            "No registered Relying Party found with client_id $client_id");
+        return undef;
+    }
+    else {
+        $self->logger->debug("Client id $client_id match Relying Party $rp");
+    }
+
+    # Check client_secret
+    if ( $self->conf->{oidcRPMetaDataOptions}->{$rp}
+        ->{oidcRPMetaDataOptionsPublic} )
+    {
+        $self->logger->debug(
+            "Relying Party $rp is public, do not check client secret");
+    }
+    else {
+        unless ($client_secret) {
+            $self->logger->error(
+"Relying Party $rp is confidential but no client secret was provided to authenticate on token endpoint"
+            );
+            return undef;
+        }
+        unless ( $client_secret eq $self->conf->{oidcRPMetaDataOptions}->{$rp}
+            ->{oidcRPMetaDataOptionsClientSecret} )
+        {
+            $self->logger->error("Wrong credentials for $rp");
+            return undef;
+        }
+    }
+    return $rp;
+}
+
 # Get Client ID and Client Secret
 # @return array (client_id, client_secret)
 sub getEndPointAuthenticationCredentials {
@@ -1047,6 +1275,10 @@ sub getEndPointAuthenticationCredentials {
         $client_id     = $req->param('client_id');
         $client_secret = $req->param('client_secret');
     }
+    elsif ( $req->param('client_id') and !$req->param('client_secret') ) {
+        $self->logger->debug("Method none used");
+        $client_id = $req->param('client_id');
+    }
 
     return ( $client_id, $client_secret );
 }
@@ -1083,22 +1315,28 @@ sub getAttributesListFromClaim {
 # @param rp Internal Relying Party identifier
 # @param user_session_id User session identifier
 # @return hashref UserInfo data
+sub buildUserInfoResponseFromId {
+    my ( $self, $req, $scope, $rp, $user_session_id ) = @_;
+    my $session = $self->p->getApacheSession($user_session_id);
+
+    return undef unless ($session);
+    return buildUserInfoResponse( $self, $req, $scope, $rp, $session );
+}
+
+# Return Hash of UserInfo data
+# @param scope OIDC scope
+# @param rp Internal Relying Party identifier
+# @param session SSO or offline session
+# @return hashref UserInfo data
 sub buildUserInfoResponse {
-    my ( $self, $scope, $rp, $user_session_id ) = @_;
+    my ( $self, $req, $scope, $rp, $session ) = @_;
     my $userinfo_response = {};
 
-    # Get user identifier
-    my $apacheSession = $self->p->getApacheSession($user_session_id);
-
-    unless ($apacheSession) {
-        $self->logger->error("Unable to find user session");
-        return undef;
-    }
     my $user_id_attribute =
       $self->conf->{oidcRPMetaDataOptions}->{$rp}
       ->{oidcRPMetaDataOptionsUserIDAttr}
       || $self->conf->{whatToTrace};
-    my $user_id = $apacheSession->data->{$user_id_attribute};
+    my $user_id = $session->data->{$user_id_attribute};
 
     $self->logger->debug("Found corresponding user: $user_id");
 
@@ -1114,7 +1352,19 @@ sub buildUserInfoResponse {
             my $session_key =
               $self->conf->{oidcRPMetaDataExportedVars}->{$rp}->{$attribute};
             if ($session_key) {
-                my $session_value = $apacheSession->data->{$session_key};
+
+                my $session_value;
+
+                # Lookup attribute in macros first
+                if ( $self->spMacros->{$rp}->{$session_key} ) {
+                    $session_value = $self->spMacros->{$rp}->{$session_key}
+                      ->( $req, $session->data );
+
+                    # If not found, search in session
+                }
+                else {
+                    $session_value = $session->data->{$session_key};
+                }
 
                 # Address is a JSON object
                 if ( $claim eq "address" ) {
@@ -1140,7 +1390,7 @@ sub createJWT {
     my ( $self, $payload, $alg, $rp ) = @_;
 
     # Payload encoding
-    my $jwt_payload = encode_base64( to_json($payload), "" );
+    my $jwt_payload = encode_base64url( to_json($payload), "" );
 
     # JWT header
     my $jwt_header_hash = { typ => "JWT", alg => $alg };
@@ -1148,7 +1398,7 @@ sub createJWT {
         $jwt_header_hash->{kid} = $self->conf->{oidcServiceKeyIdSig}
           if $self->conf->{oidcServiceKeyIdSig};
     }
-    my $jwt_header = encode_base64( to_json($jwt_header_hash), "" );
+    my $jwt_header = encode_base64url( to_json($jwt_header_hash), "" );
 
     if ( $alg eq "none" ) {
 
@@ -1161,6 +1411,11 @@ sub createJWT {
         my $client_secret =
           $self->conf->{oidcRPMetaDataOptions}->{$rp}
           ->{oidcRPMetaDataOptionsClientSecret};
+        unless ($client_secret) {
+            $self->logger->error(
+                "Algorithm $alg needs a Client Secret to sign JWT");
+            return;
+        }
 
         my $digest;
 
@@ -1182,6 +1437,7 @@ sub createJWT {
         # Convert + and / to get Base64 URL valid (RFC 4648)
         $digest =~ s/\+/-/g;
         $digest =~ s/\//_/g;
+        $digest =~ s/=+$//g;
 
         return $jwt_header . "." . $jwt_payload . "." . $digest;
     }
@@ -1190,6 +1446,12 @@ sub createJWT {
 
         # Get signing private key
         my $priv_key = $self->conf->{oidcServicePrivateKeySig};
+        unless ($priv_key) {
+            $self->logger->error(
+                "Algorithm $alg needs a Private Key to sign JWT");
+            return;
+        }
+
         my $rsa_priv = Crypt::OpenSSL::RSA->new_private_key($priv_key);
 
         if ( $alg eq "RS256" ) {
@@ -1321,7 +1583,7 @@ sub buildLogoutResponse {
 
 # Create session_state parameter
 # @param session_id Session ID
-# @param client_id CLient ID
+# @param client_id Client ID
 # return String Session state
 sub createSessionState {
     my ( $self, $session_id, $client_id ) = @_;
@@ -1371,6 +1633,16 @@ sub decode_base64url {
     return decode_base64($s);
 }
 
+sub encodeBase64url {
+    my ( $self, $value ) = @_;
+    return encode_base64url($value);
+}
+
+sub decodeBase64url {
+    my ( $self, $value ) = @_;
+    return decode_base64url($value);
+}
+
 sub addRouteFromConf {
     my ( $self, $type, %subs ) = @_;
     my $adder = "add${type}Route";
@@ -1381,10 +1653,65 @@ sub addRouteFromConf {
             $self->logger->error("$_ parameter not defined");
             next;
         }
-        $self->$adder( $self->path => { $path => $sub }, [ 'GET', 'POST' ] );
+        $self->$adder(
+            $self->path => { $path => $sub },
+            [ 'GET', 'POST' ]
+        );
     }
 }
 
+# Validate PKCE code challenge with given code challenge method
+# @param code_verifier
+# @param code_challenge
+# @param code_challenge_method
+# @return boolean 1 if challenge succeed, 0 else
+sub validatePKCEChallenge {
+    my ( $self, $code_verifier, $code_challenge, $code_challenge_method ) = @_;
+
+    unless ($code_verifier) {
+        $self->logger->error("PKCE required but no code verifier provided");
+        return 0;
+    }
+
+    $self->logger->debug("PKCE code verifier received: $code_verifier");
+
+    if ( !$code_challenge_method or $code_challenge_method eq "plain" ) {
+        if ( $code_verifier eq $code_challenge ) {
+            $self->logger->debug("PKCE challenge validated (plain method)");
+            return 1;
+        }
+        else {
+            $self->logger->error("PKCE challenge failed (plain method)");
+            return 0;
+        }
+    }
+
+    elsif ( $code_challenge_method eq "S256" ) {
+        my $code_verifier_hashed = encode_base64url( sha256($code_verifier) );
+        if ( $code_verifier_hashed eq $code_challenge ) {
+            $self->logger->debug("PKCE challenge validated (S256 method)");
+            return 1;
+        }
+        else {
+            $self->logger->error("PKCE challenge failed (S256 method)");
+            return 0;
+        }
+    }
+
+    else {
+        $self->logger->error("PKCE challenge method not valid");
+        return 0;
+    }
+
+    return 0;
+}
+
+sub force_id_claims {
+    my ( $self, $rp ) = @_;
+    return $self->conf->{oidcRPMetaDataOptions}->{$rp}
+      ->{oidcRPMetaDataOptionsIDTokenForceClaims};
+}
+
 1;
 
 __END__
@@ -1458,6 +1785,30 @@ Get UserInfo response
 
 Convert JSON to HashRef
 
+=head2 newAuthorizationCode
+
+Generate new Authorization Code session
+
+=head2 newAccessToken
+
+Generate new Access Token session
+
+=head2 newRefreshToken
+
+Generate new Refresh Token session
+
+=head2 getAuthorizationCode
+
+Get existing Authorization Code session
+
+=head2 getAccessToken
+
+Get existing Access Token session
+
+=head2 getRefreshToken
+
+Get existing Refresh Token session
+
 =head2 getOpenIDConnectSession
 
 Try to recover the OpenID Connect session corresponding to id and return session
@@ -1502,9 +1853,13 @@ Get Access Token
 
 Return list of attributes authorized for a claim
 
+=head2 buildUserInfoResponseFromId
+
+Return Hash of UserInfo data from session ID
+
 =head2 buildUserInfoResponse
 
-Return Hash of UserInfo data
+Return Hash of UserInfo data from session object
 
 =head2 createJWT
 
@@ -1543,6 +1898,10 @@ Build Logout Response URI
 Build a Lemonldap::NG::Common::PSGI::Router route from OIDC configuration
 attribute
 
+=head2 validatePKCEChallenge
+
+Validate PKCE code challenge with given code challenge method
+
 =head1 SEE ALSO
 
 L<Lemonldap::NG::Portal::AuthOpenIDConnect>, L<Lemonldap::NG::Portal::UserDBOpenIDConnect>
