@@ -8,7 +8,7 @@
 #                  of lemonldap-ng.ini) and underlying handler configuration
 package Lemonldap::NG::Portal::Main::Init;
 
-our $VERSION = '2.0.5';
+our $VERSION = '2.0.7';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -38,7 +38,7 @@ has _groups     => ( is => 'rw' );
 has _jsRedirect => ( is => 'rw' );
 
 # TrustedDomain regexp
-has trustedDomainsRe => ( is => 'rw' );
+has trustedDomainsRe         => ( is => 'rw' );
 has additionalTrustedDomains => ( is => 'rw', default => sub { [] } );
 
 # Lists to store plugins entry-points
@@ -157,6 +157,9 @@ sub init {
       # Refresh session
       ->addAuthRoute( refresh => 'refresh', ['GET'] )
 
+      ->addAuthRoute( '*' => 'corsPreflight', ['OPTIONS'] )
+      ->addUnauthRoute( '*' => 'corsPreflight', ['OPTIONS'] )
+
       # Logout
       ->addAuthRoute( logout => 'logout', ['GET'] );
 
@@ -201,7 +204,7 @@ sub reloadConf {
         my $header = $_;
         my $prm    = $self->conf->{ 'cors' . $_ };
         $header =~ s/_/-/;
-        $prm =~ s/\s+//;
+        $prm    =~ s/\s+//;
         $cors .= "Access-Control-$header;$prm;";
     }
     $self->cors($cors);
@@ -353,9 +356,23 @@ sub reloadConf {
 
     # Clean $req->pdata after authentication
     push @{ $self->endAuth }, sub {
-        unless ( $_[0]->pdata->{keepPdata} ) {
-            $self->logger->debug('Cleaning pdata');
-            $_[0]->pdata( {} );
+
+        my $tmp =
+          ( ref( $_[0]->pdata->{keepPdata} ) eq 'ARRAY' )
+          ? $_[0]->pdata->{keepPdata}
+          : [];
+        foreach my $k ( keys %{ $_[0]->pdata } ) {
+            unless ( grep { $_ eq $k } @$tmp ) {
+                $self->logger->debug("Removing $k from pdata");
+                delete $_[0]->pdata->{$k};
+            }
+        }
+        my $user_log = $_[0]->{sessionInfo}->{ $self->conf->{whatToTrace} };
+        $self->userLogger->notice( $user_log . ' connected' ) if $user_log;
+        if (@$tmp) {
+            $self->logger->debug(
+                'Add ' . join( ',', @$tmp ) . ' in keepPdata' );
+            $_[0]->pdata->{keepPdata} = $tmp;
         }
         return PE_OK;
     };
@@ -434,7 +451,7 @@ sub findEP {
         }
         else {
             foreach my $ep ( keys %$h ) {
-                my $callback = $h->{$ep};
+                my $callback    = $h->{$ep};
                 my $previousSub = $self->aroundSub->{$ep} ||= sub {
                     $self->logger->debug(
                         "$ep launched inside ${plugin}::$callback");
