@@ -1,12 +1,13 @@
 package Lemonldap::NG::Portal::Main::Process;
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.0.7';
 
 package Lemonldap::NG::Portal::Main;
 
 use strict;
 use MIME::Base64;
 use POSIX qw(strftime);
+use Lemonldap::NG::Portal::Main::Constants qw(portalConsts);
 
 # Main method
 # -----------
@@ -38,7 +39,9 @@ sub process {
             }
         }
     }
-    $self->logger->debug("Returned error: $err") if ($err);
+    $self->logger->debug(
+        "Returned error: $err (" . portalConsts->{$err} . ")" )
+      if ($err);
     return $err;
 }
 
@@ -77,7 +80,7 @@ sub controlUrl {
                 $req->set_param( 'confirm', $c );
             }
             else {
-                $self->logger->notice('Confirmation to old, refused');
+                $self->logger->notice('Confirmation too old, refused');
                 $req->set_param( 'confirm', 0 );
             }
         }
@@ -92,12 +95,18 @@ sub controlUrl {
         }
         else {
             if ( $url =~ m#[^A-Za-z0-9\+/=]# ) {
-                $self->userLogger->error(
-                    "Value must be in BASE64 (param: url | value: $url)");
-                return PE_BADURL;
+                unless ( $req->maybeNotBase64 ) {
+                    $self->userLogger->error(
+"Value must be BASE64 encoded (param: url | value: $url)"
+                    );
+                    return PE_BADURL;
+                }
+                $req->{urldc} = $url;
+            }
+            else {
+                $req->{urldc} = decode_base64($url);
+                $req->{urldc} =~ s/[\r\n]//sg;
             }
-            $req->{urldc} = decode_base64($url);
-            $req->{urldc} =~ s/[\r\n]//sg;
         }
 
         # For logout request, test if Referer comes from an authorized site
@@ -150,18 +159,22 @@ sub checkLogout {
 sub authLogout {
     my ( $self, $req ) = @_;
     my $res = $self->_authentication->authLogout($req);
-    unless ( $res or $req->pdata->{keepPdata} ) {
-        $self->logger->debug('Cleaning pdata');
-        $req->pdata( {} );
+    $self->logger->debug('Cleaning pdata');
+    my $tmp =
+      ( ref( $req->pdata->{keepPdata} ) eq 'ARRAY' )
+      ? $req->pdata->{keepPdata}
+      : [];
+    foreach my $k ( keys %{ $req->pdata } ) {
+        delete $req->pdata->{$k} unless ( grep { $_ eq $k } @$tmp );
     }
+    $req->pdata->{keepPdata} = $tmp if @$tmp;
     return $res;
 }
 
 sub deleteSession {
     my ( $self, $req ) = @_;
-    $req->userData( {} );
-    if ( my $id = $req->id ) {
-        my $apacheSession = $self->getApacheSession( $req->id );
+    if ( my $id = $req->id || $req->userData->{_session_id} ) {
+        my $apacheSession = $self->getApacheSession($id);
         unless ($apacheSession) {
             $self->logger->debug("Session $id already deleted");
             return PE_OK;
@@ -184,7 +197,8 @@ sub deleteSession {
 
         $req->info(
             $self->loadTemplate(
-                'simpleInfo', params => { trspan => 'logoutFromOtherApp' }
+                $req, 'simpleInfo',
+                params => { trspan => 'logoutFromOtherApp' }
             )
         );
 
@@ -211,6 +225,7 @@ sub deleteSession {
             $req->urldc( $req->script_name . "?logout=1" );
         }
     }
+    $req->userData( {} );
 
     # Redirect or Post if asked by authLogout
     if ( $req->urldc and $req->urldc ne $self->conf->{portal} ) {
@@ -356,6 +371,9 @@ sub setSessionInfo {
           if $self->conf->{timeoutActivity};
     }
 
+    # Currently selected language
+    $req->{sessionInfo}->{_language} = $req->cookies->{llnglanguage} || 'en';
+
     # Store URL origin in session
     $req->{sessionInfo}->{_url} = $req->{urldc};
 
@@ -404,7 +422,6 @@ sub setPersistentSessionInfo {
             }
         }
     }
-
     PE_OK;
 }
 
@@ -433,20 +450,16 @@ sub store {
     $req->userData( $req->sessionInfo );
 
     # Create second session for unsecure cookie
-    if ( $self->conf->{securedCookie} == 2 ) {
-
+    if ( $self->conf->{securedCookie} == 2 and !$req->refresh() ) {
         my %infos = %{ $req->{sessionInfo} };
+        $infos{_updateTime} = strftime( "%Y%m%d%H%M%S", localtime() );
+        $self->logger->debug("Set _updateTime with $infos{_updateTime}");
         $infos{_httpSessionType} = 1;
 
         my $session2 = $self->getApacheSession( undef, info => \%infos );
-
+        $self->logger->debug("Create second session for unsecured cookie...");
         $req->{sessionInfo}->{_httpSession} = $session2->id;
-    }
-
-    # Compute unsecure cookie value if needed
-    if ( $self->conf->{securedCookie} == 3 ) {
-        $req->{sessionInfo}->{_httpSession} =
-          $self->conf->{cipher}->encryptHex( $req->{id}, "http" );
+        $self->logger->debug( " -> Cookie value : " . $session2->id );
     }
 
     # Fill session
@@ -473,6 +486,15 @@ sub store {
     return PE_APACHESESSIONERROR unless ($session);
     $req->id( $session->{id} );
 
+    # Compute unsecured cookie value if needed
+    if ( $self->conf->{securedCookie} == 3 and !$req->refresh() ) {
+        $req->{sessionInfo}->{_httpSession} =
+          $self->conf->{cipher}->encryptHex( $req->{id}, "http" );
+        $self->logger->debug( " -> Compute unsecured cookie value : "
+              . $req->{sessionInfo}->{_httpSession} );
+    }
+    $req->refresh(0);
+
     PE_OK;
 }
 
@@ -498,6 +520,10 @@ sub buildCookie {
             );
         }
     }
+    my $user_log = $req->{userData}->{ $self->conf->{whatToTrace} };
+    $self->userLogger->notice(
+"User $user_log successfully authenticated at level $req->{userData}->{authenticationLevel}"
+    );
     PE_OK;
 }
 
