@@ -24,14 +24,18 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_PASSWORDFORMEMPTY
   PE_PASSWORD_MISMATCH
   PE_PASSWORD_OK
+  PE_PP_INSUFFICIENT_PASSWORD_QUALITY
+  PE_PP_PASSWORD_TOO_SHORT
+  PE_PP_PASSWORD_TOO_YOUNG
+  PE_PP_PASSWORD_IN_HISTORY
   PE_TOKENEXPIRED
   PE_USERNOTFOUND
 );
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.0.6';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin',
-  'Lemonldap::NG::Portal::Lib::SMTP';
+  'Lemonldap::NG::Portal::Lib::SMTP', 'Lemonldap::NG::Portal::Lib::_tokenRule';
 
 # PROPERTIES
 
@@ -84,7 +88,7 @@ sub resetPwd {
 
 sub _reset {
     my ( $self, $req ) = @_;
-    my ( $mailToken, $newPwd, $confirmPwd, %tplPrms );
+    my ( $mailToken, %tplPrms );
 
     # PASSWORD CHANGE FORM => changePwd()
     if (
@@ -109,10 +113,11 @@ sub _reset {
 
     # OTHER FORMS
     if ($mailToken) {
-        $self->logger->debug( "Token given for password reset: " . $mailToken );
+        $self->logger->debug("Token given for password reset: $mailToken");
 
         # Check if token is valid
-        my $mailSession = $self->p->getApacheSession($mailToken);
+        my $mailSession =
+          $self->p->getApacheSession( $mailToken, kind => "TOKEN" );
         unless ($mailSession) {
             $self->userLogger->warn('Bad reset token');
             return PE_BADMAILTOKEN;
@@ -137,7 +142,7 @@ sub _reset {
 
         # Check if token exists
         my $token;
-        if ( $self->conf->{requireToken} or $self->captcha ) {
+        if ( $self->ottRule->( $req, {} ) or $self->captcha ) {
             $token = $req->param('token');
             unless ($token) {
                 $self->setSecurity($req);
@@ -166,9 +171,9 @@ sub _reset {
                 $self->setSecurity($req);
                 return PE_CAPTCHAERROR;
             }
-            $self->logger->debug("Captcha code verified");
+            $self->logger->debug('Captcha code verified');
         }
-        elsif ( $self->conf->{requireToken} ) {
+        elsif ( $self->ottRule->( $req, {} ) ) {
             unless ( $self->ott->getToken($token) ) {
                 $self->setSecurity($req);
                 $self->userLogger->warn('Reset try with expired/bad token');
@@ -190,9 +195,9 @@ sub _reset {
     );
     if ( my $error = $self->p->process( $req, useMail => $searchByMail ) ) {
         if ( $error == PE_USERNOTFOUND or $error == PE_BADCREDENTIALS ) {
-            $self->userLogger->warn( "Reset asked for an unvalid user ("
+            $self->userLogger->warn( 'Reset asked for an unvalid user ('
                   . $req->param('mail')
-                  . ")" );
+                  . ')' );
 
             # To avoid mail enumeration, return OK
             # unless portalErrorOnMailNotFound is set
@@ -206,9 +211,9 @@ sub _reset {
               $self->conf->{mailTimeout} || $self->conf->{timeout};
             my $expTimestamp = time() + $mailTimeout;
             $req->data->{expMailDate} =
-              strftime( "%d/%m/%Y", localtime $expTimestamp );
+              strftime( '%d/%m/%Y', localtime $expTimestamp );
             $req->data->{expMailTime} =
-              strftime( "%H:%M", localtime $expTimestamp );
+              strftime( '%H:%M', localtime $expTimestamp );
             return PE_MAILCONFIRMOK;
         }
         return $error;
@@ -245,13 +250,14 @@ sub _reset {
         $infos->{user} = $req->{user};
 
         # Store type
-        $infos->{_type} = "mail";
+        $infos->{_type} = 'mail';
 
         # Store pdata
         $infos->{_pdata} = $req->pdata;
 
         # create session
-        $mailSession = $self->p->getApacheSession( undef, info => $infos );
+        $mailSession =
+          $self->p->getApacheSession( undef, kind => "TOKEN", info => $infos );
 
         $req->id( $mailSession->id );
     }
@@ -270,22 +276,22 @@ sub _reset {
         $self->logger->debug("Mail expiration timestamp: $expTimestamp");
 
         $req->data->{expMailDate} =
-          strftime( "%d/%m/%Y", localtime $expTimestamp );
+          strftime( '%d/%m/%Y', localtime $expTimestamp );
         $req->data->{expMailTime} =
-          strftime( "%H:%M", localtime $expTimestamp );
+          strftime( '%H:%M', localtime $expTimestamp );
 
         # Mail session start date
         my $startTimestamp = $mailSession->data->{mailSessionStartTimestamp};
 
         $self->logger->debug("Mail start timestamp: $startTimestamp");
         $req->data->{startMailDate} =
-          strftime( "%d/%m/%Y", localtime $startTimestamp );
+          strftime( '%d/%m/%Y', localtime $startTimestamp );
         $req->data->{startMailTime} =
-          strftime( "%H:%M", localtime $startTimestamp );
+          strftime( '%H:%M', localtime $startTimestamp );
 
         # Ask if user wants an another confirmation email
         if ( $req->data->{mailAlreadySent}
-            and !$req->param('resendconfirmation') )
+            and not $req->param('resendconfirmation') )
         {
             $self->userLogger->notice(
                 'Reset mail already sent to ' . $req->{user} );
@@ -332,7 +338,8 @@ sub _reset {
         else {
 
             # Use HTML template
-            $body = $self->loadTemplate(
+            $body = $self->loadMailTemplate(
+                $req,
                 'mail_confirm',
                 filter => $tr,
                 params => \%tplPrms
@@ -405,14 +412,12 @@ sub changePwd {
     # Check if user wants to generate the new password
     if ( $req->param('reset') ) {
         $self->logger->debug(
-            "Reset password request for " . $req->{sessionInfo}->{_user} );
+            "Reset password request for $req->{sessionInfo}->{_user}");
 
         # Generate a complex password
         my $password =
           $self->gen_password( $self->conf->{randomPasswordRegexp} );
-
-        $self->logger->debug( "Generated password: " . $password );
-
+        $self->logger->debug("Generated password: $password");
         $req->data->{newpassword}     = $password;
         $req->data->{confirmpassword} = $password;
         $req->data->{forceReset}      = 1;
@@ -435,6 +440,16 @@ sub changePwd {
         }
     }
 
+    # Check password quality
+    require Lemonldap::NG::Portal::Password::Base;
+    my $cpq =
+      $self->Lemonldap::NG::Portal::Password::Base::checkPasswordQuality(
+        $req->data->{newpassword} );
+    unless ( $cpq == PE_OK ) {
+        $self->ott->setToken( $req, $req->sessionInfo );
+        return $cpq;
+    }
+
     # Modify the password TODO: change this
     # Populate $req->{user} for logging purpose
     my $tmp = $self->conf->{portalRequireOldPassword};
@@ -447,7 +462,10 @@ sub changePwd {
     $self->conf->{portalRequireOldPassword} = $tmp;
 
     # Mail token can be used only one time, delete the session if all is ok
-    return $result unless ( $result == PE_PASSWORD_OK or $result == PE_OK );
+    unless ( $result == PE_PASSWORD_OK or $result == PE_OK ) {
+        $self->ott->setToken( $req, $req->sessionInfo );
+        return $result;
+    }
 
     # Send mail containing the new password
     $req->data->{mailAddress} ||=
@@ -472,7 +490,8 @@ sub changePwd {
     else {
 
         # Use HTML template
-        $body = $self->loadTemplate(
+        $body = $self->loadMailTemplate(
+            $req,
             'mail_password',
             filter => $tr,
             params => \%tplPrms
@@ -490,7 +509,7 @@ sub changePwd {
       unless $self->send_mail( $req->data->{mailAddress}, $subject, $body,
         $html );
 
-    PE_MAILOK;
+    return PE_MAILOK;
 }
 
 sub setSecurity {
@@ -498,9 +517,10 @@ sub setSecurity {
     if ( $self->captcha ) {
         $self->captcha->setCaptcha($req);
     }
-    elsif ( $self->conf->{requireToken} ) {
+    elsif ( $self->ottRule->( $req, {} ) ) {
         $self->ott->setToken($req);
     }
+    return 1;
 }
 
 sub display {
@@ -522,7 +542,7 @@ sub display {
         MAILALREADYSENT => $req->data->{mailAlreadySent},
         MAIL            => (
             $self->p->checkXSSAttack( 'mail', $req->{user} )
-            ? ""
+            ? ''
             : $req->{user}
         ),
         DISPLAY_FORM            => 0,
@@ -530,9 +550,17 @@ sub display {
         DISPLAY_CONFIRMMAILSENT => 0,
         DISPLAY_MAILSENT        => 0,
         DISPLAY_PASSWORD_FORM   => 0,
+        DISPLAY_PPOLICY         => $self->conf->{portalDisplayPasswordPolicy},
+        PPOLICY_MINSIZE         => $self->conf->{passwordPolicyMinSize},
+        PPOLICY_MINLOWER        => $self->conf->{passwordPolicyMinLower},
+        PPOLICY_MINUPPER        => $self->conf->{passwordPolicyMinUpper},
+        PPOLICY_MINDIGIT        => $self->conf->{passwordPolicyMinDigit},
+        DISPLAY_GENERATE_PASSWORD =>
+          $self->conf->{portalDisplayGeneratePassword},
     );
     if ( $req->data->{mailToken}
-        and !$self->p->checkXSSAttack( 'mail_token', $req->data->{mailToken} ) )
+        and
+        not $self->p->checkXSSAttack( 'mail_token', $req->data->{mailToken} ) )
     {
         $tplPrm{MAIL_TOKEN} = $req->data->{mailToken};
     }
@@ -554,7 +582,7 @@ sub display {
             or $req->error == PE_CAPTCHAERROR
             or $req->error == PE_CAPTCHAEMPTY
         )
-        and !$req->data->{mailToken}
+        and not $req->data->{mailToken}
       )
     {
         $self->logger->debug('Display form');
@@ -589,9 +617,15 @@ sub display {
         $tplPrm{DISPLAY_PASSWORD_FORM} = 1;
     }
 
-    # Display password change form again if passwords mismatch
+    # Display password change form again
+    # - if passwords mismatch
+    # - if password quality check fail
     elsif ($req->error == PE_PASSWORDFORMEMPTY
-        || $req->error == PE_PASSWORD_MISMATCH )
+        || $req->error == PE_PASSWORD_MISMATCH
+        || $req->error == PE_PP_INSUFFICIENT_PASSWORD_QUALITY
+        || $req->error == PE_PP_PASSWORD_TOO_SHORT
+        || $req->error == PE_PP_PASSWORD_TOO_YOUNG
+        || $req->error == PE_PP_PASSWORD_IN_HISTORY )
     {
         $self->logger->debug('Display password form');
         $tplPrm{DISPLAY_PASSWORD_FORM} = $req->sessionInfo->{pwdAllowed};
