@@ -15,8 +15,10 @@
 #   * GET /session/my/<type>                     : get session data
 #   * GET /session/my/<type>/key                 : get session key
 #   * DELETE /session/my                         : ask for logout
+#   * DELETE /sessions/my                        : ask for global logout
 #
 # - Authentication
+#   * GET /renewcaptcha                          : get token and captcha image
 #   * POST /sessions/<type>/<session-id>?auth    : authenticate with a fixed
 #                                                  sessionId
 #   * Note that the "getCookie" method (authentification via SOAP) exists for
@@ -52,9 +54,10 @@ use Mouse;
 use JSON qw(from_json to_json);
 use MIME::Base64;
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.7';
 
-extends 'Lemonldap::NG::Portal::Main::Plugin';
+extends
+  qw (Lemonldap::NG::Portal::Main::Plugin Lemonldap::NG::Portal::Lib::Captcha);
 
 has configStorage => (
     is      => 'ro',
@@ -94,8 +97,8 @@ has exportedAttr => (
         }
     }
 );
-
-has ott => (
+has captcha => ( is => 'rw' );
+has ott     => (
     is      => 'rw',
     lazy    => 1,
     default => sub {
@@ -112,6 +115,13 @@ sub init {
     my ($self)  = @_;
     my @parents = ('Lemonldap::NG::Portal::Main::Plugin');
     my $add     = 0;
+    if (   $self->conf->{captcha_mail_enabled}
+        || $self->conf->{captcha_login_enabled}
+        || $self->conf->{captcha_register_enabled} )
+    {
+        $self->captcha( $self->p->loadModule('::Lib::Captcha') ) or return 0;
+        $self->addUnauthRoute( renewcaptcha => 'sendCaptcha', ['GET'] );
+    }
     if ( $self->conf->{restConfigServer} ) {
         push @parents, 'Lemonldap::NG::Common::Conf::RESTServer';
         $add++;
@@ -170,6 +180,11 @@ sub init {
           ->addAuthRoute(
             session => { my => { ':sessionType' => 'getMyKey' } },
             [ 'GET', 'POST' ]
+          )
+
+          ->addAuthRoute(
+            sessions => { my => { ':sessionType' => 'removeSessions' } },
+            ['DELETE']
           );
     }
 
@@ -552,4 +567,22 @@ sub getError {
     );
 }
 
+sub removeSessions {
+    my ( $self, $req ) = @_;
+    my $glPlugin =
+      $self->p->loadedModules->{'Lemonldap::NG::Portal::Plugins::GlobalLogout'};
+    my $sessions = $glPlugin->activeSessions($req);
+    my $nbr      = $glPlugin->removeOtherActiveSessions( $req, $sessions );
+
+    return $self->p->sendJSONresponse( $req, { result => $nbr } );
+}
+
+sub sendCaptcha {
+    my ( $self,  $req )   = @_;
+    my ( $token, $image ) = $self->captcha->getCaptcha($req);
+
+    return $self->p->sendJSONresponse( $req,
+        { newtoken => $token, newimage => $image } );
+}
+
 1;
