@@ -1,6 +1,9 @@
 package Lemonldap::NG::Portal::Register::Custom;
 
 use strict;
+use Mouse;
+
+extends 'Lemonldap::NG::Portal::Register::Base';
 
 sub new {
     my ( $class, $self ) = @_;
