@@ -73,7 +73,8 @@ sub createUser {
     if ( $mesg->is_error ) {
         $self->userLogger->error(
             "Can not create entry for " . $req->data->{registerInfo}->{login} );
-        $self->logger->error( "LDAP error " . $mesg->error );
+        $self->logger->error(
+            "LDAP error " . $mesg->code . ": " . $mesg->error );
 
         $self->ldap->unbind();
         $self->{flags}->{ldapActive} = 0;
